from queue import Queue
from fivenines_agent.env import debug_mode

class SynchronizationQueue(Queue):
    def __init__(self, maxsize=100):
        Queue.__init__(self, maxsize)

    def put(self, data):
        with self.mutex:
            if self._qsize() > self.maxsize:
                if debug_mode():
                    print(f'Queue size too big: {self._qsize()}. Dropping oldest data')
                super()._get()
                self.unfinished_tasks -= 1

            super()._put(data)
            self.unfinished_tasks += 1
            self.not_empty.notify()

    def clear(self):
        with self.mutex:
            self.queue.clear()
            self.unfinished_tasks = 0
            self.not_full.notify() 
            self.all_tasks_done.notify_all()



import os
import sys
import traceback

METRICS = '\|'.join([
  'redis_version',
  'connected_clients',
  'maxclients',
  '^db[0-9]'
])

def redis_metrics(port=6379, password=None):
    redis_installed = False
    if os.system('which redis-server > /dev/null') == 0:
      redis_installed = True

    if not redis_installed:
      return None

    auth_prefix = ''
    if password:
      auth_prefix = f'AUTH {password}\n'

    try:
      with os.popen(f'echo "{auth_prefix}INFO\nQUIT" | curl -s telnet://localhost:{port} | grep -e "{METRICS}"', 'r') as f:
        results = list(filter(None, f.read().rstrip('\n').split('\n')))

      metrics = {}

      if len(results) > 0:
        for result in results:
          key, value = result.split(':')
          if key == 'redis_version':
            metrics[key] = value.strip()
          elif key.startswith('db'):
            metrics[key] = {}
            values = value.split(',')
            for v in values:
              k, v = v.split('=')
              metrics[key][k] = int(v.strip())
          else:
            metrics[key] = int(value.strip())

      return metrics

    except Exception as e:
      print(e, file=sys.stderr)
      print(traceback.print_exc(), file=sys.stderr)



import psutil
import os
import platform

def cpu_data():
    cpu_times_percent = psutil.cpu_times_percent(percpu=True)
    cpu_percent = psutil.cpu_percent(percpu=True)
    cores_usage = []

    for i, v in enumerate(cpu_times_percent):
      _cpu_usage = { 'percentage': cpu_percent[i] }
      _cpu_usage.update(v._asdict())
      cores_usage.append(_cpu_usage)

    return cores_usage


def cpu_model():
    operating_system = platform.system()
    if operating_system == 'Linux':
        try:
            with open('/proc/cpuinfo', 'r') as f:
                for line in f:
                    if line.startswith('model name'):
                        return line.split(':')[1].strip()
        except FileNotFoundError:
            return '-'
    elif operating_system == 'Darwin':
        try:
            with os.popen('/usr/sbin/sysctl -n machdep.cpu.brand_string') as f:
                return f.read().strip()
        except FileNotFoundError:
            return '-'
    else:
        '-'



import platform

def file_handles_used():
    file_handles_stats()[0]

def file_handles_limit():
    file_handles_stats()[2]


def file_handles_stats():
    operating_system = platform.system()

    if operating_system != 'Linux':
        return [0, 0, 0]
    else:
        try:
            with open('/proc/sys/fs/file-nr', 'r') as f:
                return list(map(int, f.read().strip().split('\t')))
        except FileNotFoundError:
            return [0, 0, 0]



import time
import http.client
import json
from threading import Thread
from threading import Lock

from fivenines_agent.env import debug_mode, api_url

class Synchronizer(Thread):
    def __init__(self, token, queue):
        Thread.__init__(self)
        self.config_lock = Lock()
        self.token = token
        self.config = { 'enabled': True, 'request_options': { 'timeout': 5, 'retry': 3, 'retry_interval': 5 } }
        self.queue = queue

        self.send_request({'get_config': True})

    def run(self):
        while True:
            data = self.queue.get()
            if data == None:
                break

            self.send_request(data)
            self.queue.task_done()

    def send_request(self, data):
        try_count = 0

        while try_count < self.config['request_options']['retry']:
            try:
                start_time = time.monotonic()
                conn = http.client.HTTPSConnection(
                    api_url(), timeout=self.config['request_options']['timeout'])
                res = conn.request('POST', '/collect', json.dumps(data), {
                                    'Authorization': f'Bearer {self.token}', 'Content-Type': 'application/json'})
                res = conn.getresponse()
                body = res.read().decode("utf-8")

                if res.status == 200:
                    if debug_mode():
                        print(f'Sync time: {time.monotonic() - start_time}')
                    config = json.loads(body)['config']
                    with self.config_lock:
                        self.config = config
                    break
                else:
                    raise Exception(f'HTTP {res.status}: {body}')
            except Exception as e:
                try_count += 1
                print(f'Synchronizer Error: {e}')
                print(f'Retrying in {self.config["request_options"]["retry_interval"] * try_count} seconds')
                time.sleep(self.config['request_options']['retry_interval'] * try_count)

    def get_config(self):
        with self.config_lock:
            return self.config
